module Deprecation
  module Notifier
    extend ActiveSupport::Concern

    class << self
      def __deprecation_logger__(class_name, method_name, options = {})
        options = options.symbolize_keys

        if options[:at].nil? || options[:at].to_time <= Time.now
          message = class_name
          message << "#" << method_name if method_name
          message << " is deprecated"
          message << (options[:at].present?? " as of #{options[:at]}." : ".")
          message << " " << options[:message] if options[:message]

          ActiveSupport::Deprecation.warn(message, caller[1..-1])
        end
      end
    end

    module ClassMethods
      def deprecate(*args)
        return if defined?(Rails) && Rails.env.production?

        if args.size == 0 || args.first.is_a?(Hash)
          deprecate_class(*args)
        else
          deprecate_method(*args)
        end
      end

      private

      def deprecate_class(options = {})
        Deprecation::Notifier.__deprecation_logger__(self.class.name, nil, options)
      end

      def deprecate_method(method_name, options = {})
        method = instance_method(method_name)
        define_method(method_name) do |*args, &block|
          Deprecation::Notifier.__deprecation_logger__(self.class.name, method_name.to_s, options)
          method.bind(self).(*args, &block)
        end
      end
    end
  end
end
